from pathlib import Path
from typing import List

import numpy as np
import tifffile as tf
from attrs import define, field, validators
from cenfind.core.log import get_logger

logger = get_logger(__name__)


def path_exists(instance, attribute, value):
    if not value.exists():
        raise FileNotFoundError(f"Invalid path ({value})")


def has_projections(instance, attribute, value):
    projections = value / "projections"
    if not projections.is_dir():
        raise FileNotFoundError(f"Invalid path for Projections: ({value}).")


def is_tif(instance, attribute, value):
    if value.suffix != ".tif":
        raise ValueError(f"Not a tif file ({value})")


@define
class Field:
    """
    Represents a Field

    It is responsible for returning the field's name and to load the corresponding image.

    Attributes:
        path: path to the image file
    """

    path: Path = field(validator=[validators.instance_of(Path), path_exists, is_tif])

    @property
    def name(self) -> str:
        """
        Extracts the field name as a string without the extension.
        """
        return self.path.stem

    @property
    def data(self) -> np.ndarray:
        """
        Loads data from path as a numpy array using `tifffile.imread`.
        """
        return tf.imread(str(self.path))


@define
class Dataset:
    """
    Represents a Dataset folder.

    It iterates over the fields and provides a setup method to create the necessary subdirectories.

    Attributes:
       path: Path to the dataset folder.
    """
    path: Path = field(validator=[
        validators.instance_of(Path),
        path_exists,
        has_projections])

    @property
    def logs(self):
        return self.path / "logs"

    @property
    def projections(self):
        return self.path / "projections"

    @property
    def predictions(self):
        return self.path / "predictions"

    @property
    def annotations(self):
        return self.path / "annotations"

    @property
    def visualisation(self):
        return self.path / "visualisation"

    @property
    def statistics(self):
        return self.path / "statistics"

    @property
    def nuclei(self):
        return self.predictions / "nuclei"

    @property
    def centrioles(self):
        return self.predictions / "centrioles"

    @property
    def cilia(self):
        return self.predictions / "cilia"

    @property
    def assignment(self):
        return self.predictions / "assignment"

    def setup(self) -> None:
        """
        Creates the subdirectories inside the Dataset path if not existing.
        The directories visualisation, statistics, predictions,
        nuclei, centrioles, cilia and assignment are created.
        """

        self.logs.mkdir(exist_ok=True)
        self.visualisation.mkdir(exist_ok=True)
        self.statistics.mkdir(exist_ok=True)
        self.predictions.mkdir(exist_ok=True)
        self.annotations.mkdir(exist_ok=True)
        self.nuclei.mkdir(exist_ok=True)
        self.centrioles.mkdir(exist_ok=True)
        self.cilia.mkdir(exist_ok=True)
        self.assignment.mkdir(exist_ok=True)

    @property
    def fields(self) -> List[Field]:
        """
        Collects all Fields found in `projections` into a list and raises a ValueError if no TIF file.
        """

        result = []
        for path in self.projections.iterdir():
            if (path.suffix == '.tif') and not path.name.startswith("."):
                result.append(Field(path))

        if len(result) == 0:
            raise ValueError(f"No field found in {self.projections}")

        return result



import pandas as pd
from pathlib import Path
import seaborn as sns
from matplotlib import pyplot as plt


def main():
    sns.set_theme()
    working_dir = Path('/Users/buergy/Downloads/IF_2022_09_07_U2OS_WTasync')
    # Concatenate the predictions and keep identity of field of view
    files = Path(working_dir / 'predictions/centrioles').glob('*.tsv')
    dfs = list()
    for f in files:
        data = pd.read_csv(f, sep='\t')
        data['file'] = f.stem
        dfs.append(data)

    df = pd.concat(dfs, ignore_index=True)

    # Plot the intensities
    g = sns.displot(df, x='intensity', hue='channel')
    g.savefig(working_dir / 'visualisation/test.png')

    # Plot the scores
    data = pd.read_csv(working_dir / 'statistics/statistics.tsv', sep='\t')
    summary = data.groupby('channel').sum(numeric_only=True)
    summary = pd.wide_to_long(summary.reset_index(), stubnames='', i='channel', j='number', suffix='(\d)|\+')
    summary.columns = ['frequency']
    summary = summary.reset_index()
    g = sns.catplot(summary, x='number', y='frequency', hue='channel', kind='bar')
    g.savefig(working_dir / 'visualisation/test2.png')

    # Plot SA/I of nuclei
    files = Path(working_dir / 'predictions/nuclei').glob('*.json')
    dfs = list()
    for f in files:
        data = pd.read_json(f, orient='index')
        data = data.drop('contour', axis=1)
        data['file'] = f.stem
        dfs.append(data)
    df = pd.concat(dfs, ignore_index=True)
    df = df.loc[df['is_nucleus_full']]

    f, ax = plt.subplots(figsize=(6.5, 6.5))
    sns.scatterplot(data=df, x='surface_area', y='intensity', ax=ax)
    sns.rugplot(data=df, x='surface_area', y='intensity', ax=ax)
    f.savefig(working_dir / 'visualisation/test3.png')

    f, ax = plt.subplots(figsize=(6.5, 6.5))
    sns.histplot(data=df, x='intensity', ax=ax, kde=True)
    f.savefig(working_dir / 'visualisation/test4.png')

    print(df)


if __name__ == '__main__':
    main()
